package net.knarcraft.minecraftserverlauncher.userinterface;

import net.knarcraft.minecraftserverlauncher.Main;
import net.knarcraft.minecraftserverlauncher.profile.ServerLauncherController;
import net.knarcraft.minecraftserverlauncher.server.ServerHandler;
import net.knarcraft.minecraftserverlauncher.utility.BackupUtil;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.concurrent.Executors;

public class ControlPanelTab implements ActionListener {

    private JButton startServerButton;
    private JButton stopServerButton;
    private JButton backupButton;
    private JButton addProfileButton;
    private JButton deleteProfileButton;
    private JComboBox<String> profiles;
    private final JPanel controlPanelPanel;
    private final ServerLauncherController controller;
    private final JLabel lblStatusLabel = new JLabel("Servers are stopped");

    public ControlPanelTab(JPanel controlPanelPanel) {
        this.controlPanelPanel = controlPanelPanel;
        this.controller = Main.getController();
        initialize();
    }

    /**
     * Sets the text of the status label
     * @param text <p>The new text of the status label</p>
     */
    public void setStatusText(String text) {
        this.lblStatusLabel.setText(text);
    }

    /**
     * Updates the ServerLauncherGUI components to block a user from doing illegal actions
     *
     * @param running <p>Whether the servers are currently running</p>
     */
    public void updateGUIElementsWhenServersStartOrStop(boolean running) {
        boolean stopped = !running; //Most gui is only enabled when the server is stopped rather than running.
        profiles.setEnabled(stopped);
        addProfileButton.setEnabled(stopped);
        deleteProfileButton.setEnabled(stopped);
        startServerButton.setEnabled(stopped);
        stopServerButton.setEnabled(running);
    }

    /**
     * Stops all servers
     */
    public void stopServers() {
        GUI gui = Main.getController().getGUI();
        try {
            gui.setStatus("Servers are stopping...");
            ServerHandler.stop();
        } catch (IOException e1) {
            gui.showError("Could not stop server.");
            e1.printStackTrace();
        } catch (InterruptedException e) {
            gui.showError("Could not kill server.");
            e.printStackTrace();
        }
    }

    /**
     * Gets the currently selected profile
     *
     * @return <p>The currently selected profile or null</p>
     */
    public String getSelectedProfile() {
        Object selectedProfile = profiles.getSelectedItem();
        if (selectedProfile != null) {
            return selectedProfile.toString();
        } else {
            return null;
        }

    }

    /**
     * Updates the profiles combo
     */
    public void updateProfiles() {
        String selectedProfile = Main.getController().getCurrentProfile().getName();
        this.profiles.removeAllItems();
        for (String profile : Main.getController().getProfileNames()) {
            this.profiles.addItem(profile);
        }
        this.profiles.setSelectedItem(selectedProfile);
    }

    /**
     * Deletes the selected profile
     */
    private void deleteProfile() {
        Object selected = profiles.getSelectedItem();
        int answer = JOptionPane.showConfirmDialog(null,
                String.format("Do you really want to remove the profile %s?", selected), "Remove profile",
                JOptionPane.YES_NO_OPTION
        );
        if (answer == JOptionPane.YES_NO_OPTION) {
            if (selected != null) {
                controller.removeProfile(selected.toString());
                updateProfiles();
            }
        }
    }

    /**
     * Saves the previous profile and loads data from the new profile
     */
    private void changeProfile() {
        controller.saveState();
        Object current = this.profiles.getSelectedItem();
        if (current != null) {
            controller.setCurrentProfile(current.toString());
        }
        controller.getGUI().updateWithSavedProfileData();
        controller.getCurrentProfile().updateConsoles();
    }

    private void initialize() {
        SpringLayout sl_panel = new SpringLayout();
        controlPanelPanel.setLayout(sl_panel);

        JLabel lblBasicControls = new JLabel("Basic controls");
        sl_panel.putConstraint(SpringLayout.NORTH, lblBasicControls, 10, SpringLayout.NORTH, controlPanelPanel);
        controlPanelPanel.add(lblBasicControls);

        startServerButton = new JButton("Start servers");
        sl_panel.putConstraint(SpringLayout.WEST, lblBasicControls, 0, SpringLayout.WEST, startServerButton);
        sl_panel.putConstraint(SpringLayout.NORTH, startServerButton, 6, SpringLayout.SOUTH, lblBasicControls);
        sl_panel.putConstraint(SpringLayout.WEST, startServerButton, 10, SpringLayout.WEST, controlPanelPanel);
        controlPanelPanel.add(startServerButton);
        startServerButton.addActionListener(this);

        stopServerButton = new JButton("Stop servers");
        sl_panel.putConstraint(SpringLayout.NORTH, stopServerButton, 0, SpringLayout.NORTH, startServerButton);
        sl_panel.putConstraint(SpringLayout.WEST, stopServerButton, 6, SpringLayout.EAST, startServerButton);
        controlPanelPanel.add(stopServerButton);
        stopServerButton.addActionListener(this);

        JLabel lblProfile = new JLabel("Profile");
        sl_panel.putConstraint(SpringLayout.NORTH, lblProfile, 6, SpringLayout.SOUTH, startServerButton);
        sl_panel.putConstraint(SpringLayout.WEST, lblProfile, 10, SpringLayout.WEST, controlPanelPanel);
        controlPanelPanel.add(lblProfile);

        addProfileButton = new JButton("+");
        sl_panel.putConstraint(SpringLayout.NORTH, addProfileButton, 6, SpringLayout.SOUTH, lblProfile);
        sl_panel.putConstraint(SpringLayout.WEST, addProfileButton, 10, SpringLayout.WEST, controlPanelPanel);
        controlPanelPanel.add(addProfileButton);
        addProfileButton.addActionListener(this);

        deleteProfileButton = new JButton("-");
        sl_panel.putConstraint(SpringLayout.NORTH, deleteProfileButton, 0, SpringLayout.NORTH, addProfileButton);
        sl_panel.putConstraint(SpringLayout.WEST, deleteProfileButton, 6, SpringLayout.EAST, addProfileButton);
        controlPanelPanel.add(deleteProfileButton);
        deleteProfileButton.addActionListener(this);

        profiles = new JComboBox<>();
        sl_panel.putConstraint(SpringLayout.NORTH, profiles, 0, SpringLayout.NORTH, addProfileButton);
        sl_panel.putConstraint(SpringLayout.WEST, profiles, 6, SpringLayout.EAST, deleteProfileButton);
        sl_panel.putConstraint(SpringLayout.EAST, profiles, 124, SpringLayout.EAST, deleteProfileButton);
        controlPanelPanel.add(profiles);
        profiles.addActionListener(this);

        sl_panel.putConstraint(SpringLayout.NORTH, lblStatusLabel, 6, SpringLayout.SOUTH, addProfileButton);
        sl_panel.putConstraint(SpringLayout.SOUTH, lblStatusLabel, -10, SpringLayout.SOUTH, controlPanelPanel);
        sl_panel.putConstraint(SpringLayout.WEST, lblStatusLabel, 10, SpringLayout.WEST, controlPanelPanel);
        sl_panel.putConstraint(SpringLayout.EAST, lblStatusLabel, -10, SpringLayout.EAST, controlPanelPanel);
        controlPanelPanel.add(lblStatusLabel);

        backupButton = new JButton("Backup");
        sl_panel.putConstraint(SpringLayout.NORTH, backupButton, 0, SpringLayout.NORTH, startServerButton);
        sl_panel.putConstraint(SpringLayout.WEST, backupButton, 6, SpringLayout.EAST, stopServerButton);
        controlPanelPanel.add(backupButton);
        backupButton.addActionListener(this);
    }

    @Override
    public void actionPerformed(ActionEvent actionEvent) {
        Object actionSource = actionEvent.getSource();
        if (actionSource == startServerButton) {
            controller.saveState();
            Executors.newSingleThreadExecutor().execute(ServerHandler::startServers);
        } else if (actionSource == stopServerButton) {
            stopServers();
        } else if (actionSource == backupButton) {
            //Run backup in its own thread to prevent locking up
            Executors.newSingleThreadExecutor().execute(() -> BackupUtil.backup(controller.getGUI()));
        } else if (actionSource == addProfileButton) {
            controller.addProfile(JOptionPane.showInputDialog("Profile name: "));
            updateProfiles();
        } else if (actionSource == deleteProfileButton) {
            deleteProfile();
        } else if (actionSource == profiles) {
            changeProfile();
        }
    }
}
